package com.tzld.piaoquan.api.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.api.common.enums.SecretEnum;
import com.tzld.piaoquan.api.model.vo.AccessTokenParam;
import com.tzld.piaoquan.api.model.vo.AccessTokenVo;
import com.tzld.piaoquan.common.common.base.CommonResponse;
import com.tzld.piaoquan.common.common.enums.ExceptionCodeEnum;
import com.tzld.piaoquan.common.component.HttpPoolClient;
import com.tzld.piaoquan.api.dao.mapper.CorpMapper;
import com.tzld.piaoquan.common.model.po.Corp;
import com.tzld.piaoquan.common.model.po.CorpExample;
import com.tzld.piaoquan.api.service.AccessTokenService;
import com.tzld.piaoquan.common.utils.LarkRobotUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

import static com.tzld.piaoquan.common.common.constant.RedisConstant.WE_COM_ACCESS_TOKEN;
import static com.tzld.piaoquan.common.common.constant.WeComConstant.GET_WE_COM_ACCESS_TOKEN_URL;

@Slf4j
@Service
public class AccessTokenServiceImpl implements AccessTokenService {

    private static final String ACCESS_TOKEN_LIST = "access_token_list";

    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private RedisTemplate<String, String> stringRedisTemplate;

    @Autowired
    private CorpMapper corpMapper;

    @Override
    public String getWeComAccessToken(Long corpId) {
        try {
            String key = String.format(WE_COM_ACCESS_TOKEN, corpId);
            String accessToken = (String) redisTemplate.opsForValue().get(key);
            if (StringUtils.isNotEmpty(accessToken)) {
                return accessToken;
            }
            CorpExample example = new CorpExample();
            example.createCriteria().andIdEqualTo(corpId);
            List<Corp> corps = corpMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(corps)) {
                LarkRobotUtil.sendMessage("查询不到主体，获取access_token失败");
                return "";
            }
            Corp corp = corps.get(0);
            String cropId = corp.getCorpId();
            String secret = corp.getSecret();
            String res = httpPoolClient.get(String.format(GET_WE_COM_ACCESS_TOKEN_URL +
                    "?corpid=%s&corpsecret=%s", cropId, secret));
            JSONObject jsonObject = JSONObject.parseObject(res);
            Long expiresIn = jsonObject.getLong("expires_in");
            String newAccessToken = jsonObject.getString("access_token");
            redisTemplate.opsForValue().set(key, newAccessToken, expiresIn, TimeUnit.SECONDS);
            return newAccessToken;
        } catch (IOException e) {
            LarkRobotUtil.sendMessage("getWeComAccessToken error:" + e);
            log.error("getWeComAccessToken error", e);
        }
        return "";
    }

    @Override
    public CommonResponse<AccessTokenVo> getAccessToken(AccessTokenParam param) throws RuntimeException {
        if (param == null || StringUtils.isEmpty(param.getSecret())) {
            return CommonResponse.create(ExceptionCodeEnum.PARAMS_ERROR, "未找到 secret");
        }
        if (!SecretEnum.contains(param.getSecret())) {
            return CommonResponse.create(ExceptionCodeEnum.PARAMS_ERROR, "secret 不存在");
        }
        AccessTokenVo accessTokenVo = new AccessTokenVo();
        String accessToken = (String) stringRedisTemplate.opsForValue().get(param.getSecret());
        if (StringUtils.isNotEmpty(accessToken)) {
            Long expire = Long.parseLong(String.valueOf(stringRedisTemplate.getExpire(param.getSecret(), TimeUnit.SECONDS)));
            accessTokenVo.setAccessToken(accessToken);
            accessTokenVo.setExpires(expire);
            if (stringRedisTemplate.opsForHash().size(ACCESS_TOKEN_LIST) > 20) {
                Map<Object, Object> entries = stringRedisTemplate.opsForHash().entries(ACCESS_TOKEN_LIST);
                entries.forEach((k, v) -> {
                    long timestamp = Long.parseLong((String) v);
                    if (System.currentTimeMillis() / 1000 > timestamp) {
                        stringRedisTemplate.opsForHash().delete(ACCESS_TOKEN_LIST, k);
                    }
                });
            }
        } else {
            String newAccessToken = UUID.randomUUID().toString().replace("-", "");
            stringRedisTemplate.opsForValue().set(param.getSecret(), newAccessToken, 2L, TimeUnit.HOURS);
            stringRedisTemplate.opsForValue().set(newAccessToken, param.getSecret(), 3L, TimeUnit.HOURS);
            long expire = 7200L;
            String timestamp = String.valueOf(System.currentTimeMillis() / 1000 + expire + 600);
            stringRedisTemplate.opsForHash().put(ACCESS_TOKEN_LIST, newAccessToken, timestamp);
            accessTokenVo.setAccessToken(newAccessToken);
            accessTokenVo.setExpires(expire);
        }
        return CommonResponse.success(accessTokenVo);
    }

    @Override
    public boolean validateAccessToken(String accessToken) {
        Object o = stringRedisTemplate.opsForHash().get(ACCESS_TOKEN_LIST, accessToken);
        if (o == null) {
            return false;
        }
        long timestamp = Long.parseLong((String) o);
        if (System.currentTimeMillis() / 1000 > timestamp) {
            stringRedisTemplate.opsForHash().delete(ACCESS_TOKEN_LIST, accessToken);
            return false;
        }
        return true;
    }

    @Override
    public SecretEnum getSecretEnum(String accessToken) {
        String secret = (String) stringRedisTemplate.opsForValue().get(accessToken);
        if(StringUtils.isEmpty(secret)){
            return null;
        }
        return SecretEnum.get(secret);
    }
}
